#ifndef CLASS_PCB_PCB_LAYER_BOX_SELECTOR_H
#define CLASS_PCB_PCB_LAYER_BOX_SELECTOR_H 1

#include <class_layer_box_selector.h>

class PCB_BASE_FRAME;

/**
 * Class to display a pcb layer list in a wxBitmapComboBox.
 */
class PCB_LAYER_BOX_SELECTOR : public LAYER_BOX_SELECTOR
{
    PCB_BASE_FRAME* m_boardFrame;

    LSET m_layerMaskDisable;    // A mask to remove some layers from layer list

public:
    // Hotkey Info
    EDA_HOTKEY_CONFIG* m_hotkeys;

public:
    // If you are thinking the constructor is a bit curious,
    // just remember it is used by automaticallty generated by wxFormBuilder files,
    // and it should mimic the wxBitmapComboBox constructor.
    // Therefore, value, style are not yet used,
    // but they are here for compatibility
    PCB_LAYER_BOX_SELECTOR( wxWindow* parent, wxWindowID id,
                        const wxString& value = wxEmptyString,
                        const wxPoint& pos = wxDefaultPosition,
                        const wxSize& size = wxDefaultSize,
                        int n = 0, const wxString choices[] = NULL, int style = 0 ) :
        LAYER_BOX_SELECTOR( parent, id, pos, size, n, choices )
    {
        m_boardFrame = NULL;
    }

    // Accessors

    // SetBoardFrame should be called after creating a PCB_LAYER_BOX_SELECTOR
    // It is not passed through the constructor because when using wxFormBuilder
    // we should use a constructor compatible with a wxBitmapComboBox
    void SetBoardFrame( PCB_BASE_FRAME* aFrame ) { m_boardFrame = aFrame; };

    // SetLayerSet allows disableing some layers, which are not
    // shown in list;
    void SetLayerSet( LSET aMask ) { m_layerMaskDisable = aMask; }

    // Reload the Layers names and bitmaps
    // Virtual function
    void Resync();

private:
    // Returns a color index from the layer id
    // Virtual function
    EDA_COLOR_T GetLayerColor( LAYER_NUM aLayer ) const;

    // Returns true if the layer id is enabled (i.e. is it should be displayed)
    // Virtual function
    bool IsLayerEnabled( LAYER_NUM aLayer ) const;

    // Returns the name of the layer id
    // Virtual function
    wxString GetLayerName( LAYER_NUM aLayer ) const;

    LSET getEnabledLayers() const;
};

#endif // CLASS_PCB_PCB_LAYER_BOX_SELECTOR_H
